"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.flowsEndpoint = exports.generateFlowsList = void 0;
const crypto_1 = require("crypto");
const fs_1 = __importDefault(require("fs"));
const zod_to_json_schema_1 = __importDefault(require("zod-to-json-schema"));
const guards_1 = require("./guards");
const get_step_language_1 = require("./get-step-language");
const generateFlowsList = (flows) => {
    const list = [];
    Object.keys(flows).forEach((flowId) => {
        const steps = [];
        const flowSteps = flows[flowId].steps;
        flowSteps.forEach((step) => {
            const filePathWithoutExtension = step.filePath.replace(/\.[^/.]+$/, '');
            const tsxPath = filePathWithoutExtension + '.tsx';
            const nodeComponentPath = fs_1.default.existsSync(tsxPath) ? tsxPath : undefined;
            if ((0, guards_1.isApiStep)(step)) {
                steps.push({
                    id: (0, crypto_1.randomUUID)(),
                    type: 'api',
                    name: step.config.name,
                    description: step.config.description,
                    emits: [...step.config.emits, ...(step.config.virtualEmits ?? [])],
                    subscribes: step.config.virtualSubscribes ?? undefined,
                    action: 'webhook',
                    language: (0, get_step_language_1.getStepLanguage)(step.filePath),
                    webhookUrl: `${step.config.method} ${step.config.path}`,
                    bodySchema: step?.config.bodySchema ? (0, zod_to_json_schema_1.default)(step.config.bodySchema) : undefined,
                    nodeComponentPath,
                });
            }
            else if ((0, guards_1.isEventStep)(step)) {
                steps.push({
                    id: (0, crypto_1.randomUUID)(),
                    type: 'event',
                    name: step.config.name,
                    description: step.config.description,
                    emits: [...step.config.emits, ...(step.config.virtualEmits ?? [])],
                    subscribes: step.config.subscribes,
                    language: (0, get_step_language_1.getStepLanguage)(step.filePath),
                    nodeComponentPath,
                });
            }
            else if ((0, guards_1.isNoopStep)(step)) {
                steps.push({
                    id: (0, crypto_1.randomUUID)(),
                    type: 'noop',
                    name: step.config.name,
                    description: step.config.description,
                    emits: step.config.virtualEmits,
                    subscribes: step.config.virtualSubscribes,
                    nodeComponentPath,
                });
            }
        });
        list.push({ id: flowId, name: flowId, steps });
    });
    return list;
};
exports.generateFlowsList = generateFlowsList;
const flowsEndpoint = (flows, app) => {
    const list = (0, exports.generateFlowsList)(flows);
    app.get('/flows', async (_, res) => {
        res.status(200).send(list.map(({ id, name }) => ({ id, name })));
    });
    app.get('/flows/:id', async (req, res) => {
        const { id } = req.params;
        const flow = list.find((flow) => flow.id === id);
        if (!flow) {
            res.status(404).send({ error: 'Flow not found' });
        }
        else {
            res.status(200).send(flow);
        }
    });
};
exports.flowsEndpoint = flowsEndpoint;
